@@ -1,5 +1,3 @@
-# coding: utf-8
-#
 # Copyright 2011 Yesudeep Mangalapilly <yesudeep@gmail.com>
 # Copyright 2012 Google, Inc & contributors.
 #
@@ -52,17 +50,25 @@ Class          Platforms                        Note
 
 """
 
+from __future__ import annotations
+
+import sys
 import warnings
-from watchdog.utils import platform
+
 from watchdog.utils import UnsupportedLibc
 
-if platform.is_linux():
+from .api import BaseObserverSubclassCallable
+
+Observer: BaseObserverSubclassCallable
+
+
+if sys.platform.startswith("linux"):
     try:
         from .inotify import InotifyObserver as Observer
     except UnsupportedLibc:
         from .polling import PollingObserver as Observer
 
-elif platform.is_darwin():
+elif sys.platform.startswith("darwin"):
     try:
         from .fsevents import FSEventsObserver as Observer
     except Exception:
@@ -71,16 +77,18 @@ elif platform.is_darwin():
             warnings.warn("Failed to import fsevents. Fall back to kqueue")
         except Exception:
             from .polling import PollingObserver as Observer
+
             warnings.warn("Failed to import fsevents and kqueue. Fall back to polling.")
 
-elif platform.is_bsd():
+elif sys.platform in ("dragonfly", "freebsd", "netbsd", "openbsd", "bsd"):
     from .kqueue import KqueueObserver as Observer
 
-elif platform.is_windows():
+elif sys.platform.startswith("win"):
     try:
         from .read_directory_changes import WindowsApiObserver as Observer
     except Exception:
         from .polling import PollingObserver as Observer
+
         warnings.warn("Failed to import read_directory_changes. Fall back to polling.")
 
 else:
