@@ -1,16 +1,14 @@
-from functools import partial
+from __future__ import annotations
+
+import contextlib
 import gc
 import os
 import threading
+from functools import partial
+
 import pytest
-from . import shell
 
-
-@pytest.fixture()
-def tmpdir(request):
-    path = os.path.realpath(shell.mkdtemp())
-    yield path
-    shell.rm(path, recursive=True)
+from .utils import ExpectEvent, Helper, P, StartWatching, TestEventQueue
 
 
 @pytest.fixture()
@@ -31,7 +29,9 @@ def no_thread_leaks():
     old_thread_count = threading.active_count()
     yield
     gc.collect()  # Clear the stuff from other function-level fixtures
-    assert threading.active_count() == old_thread_count  # Only previously existing threads
+    assert (
+        threading.active_count() == old_thread_count
+    )  # Only previously existing threads
 
 
 @pytest.fixture(autouse=True)
@@ -54,3 +54,29 @@ def no_warnings(recwarn):
         warnings.append("{w.filename}:{w.lineno} {w.message}".format(w=warning))
     print(warnings)
     assert not warnings
+
+
+@pytest.fixture(name="helper")
+def helper_fixture(tmpdir):
+    with contextlib.closing(Helper(tmp=os.fspath(tmpdir))) as helper:
+        yield helper
+
+
+@pytest.fixture(name="p")
+def p_fixture(helper: Helper) -> P:
+    return helper.joinpath
+
+
+@pytest.fixture(name="event_queue")
+def event_queue_fixture(helper: Helper) -> TestEventQueue:
+    return helper.event_queue
+
+
+@pytest.fixture(name="start_watching")
+def start_watching_fixture(helper: Helper) -> StartWatching:
+    return helper.start_watching
+
+
+@pytest.fixture(name="expect_event")
+def expect_event_fixture(helper: Helper) -> ExpectEvent:
+    return helper.expect_event
