@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-#
 # Copyright 2014 Thomas Amland <thomas.amland@gmail.com>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,14 +12,32 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 from time import time
+
+import pytest
+
 from watchdog.utils.delayed_queue import DelayedQueue
 
 
-def test_get():
-    q = DelayedQueue(2)
-    q.put("")
+@pytest.mark.flaky(max_runs=5, min_passes=1)
+def test_delayed_get():
+    q = DelayedQueue[str](2)
+    q.put("", True)
     inserted = time()
     q.get()
     elapsed = time() - inserted
-    assert 2.01 > elapsed > 1.99
+    # 2.10 instead of 2.05 for slow macOS slaves on Travis
+    assert 2.10 > elapsed > 1.99
+
+
+@pytest.mark.flaky(max_runs=5, min_passes=1)
+def test_nondelayed_get():
+    q = DelayedQueue[str](2)
+    q.put("", False)
+    inserted = time()
+    q.get()
+    elapsed = time() - inserted
+    # Far less than 1 second
+    assert elapsed < 1
